/*
 * Copyright (C) 2015, Charles University in Prague.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package cz.cuni.mff.d3s.stdynpor.jpf;

import gov.nasa.jpf.Config;
import gov.nasa.jpf.vm.VM;
import gov.nasa.jpf.vm.SystemState;
import gov.nasa.jpf.vm.ThreadInfo;
import gov.nasa.jpf.vm.ElementInfo;
import gov.nasa.jpf.vm.MethodInfo;
import gov.nasa.jpf.vm.Instruction;
import gov.nasa.jpf.vm.DefaultSchedulerFactory;
import gov.nasa.jpf.vm.ChoiceGenerator;
import gov.nasa.jpf.vm.choice.ThreadChoiceFromSet;
import gov.nasa.jpf.util.StringSetMatcher;
import gov.nasa.jpf.jvm.bytecode.ArrayElementInstruction;
import gov.nasa.jpf.jvm.bytecode.ArrayLoadInstruction;
import gov.nasa.jpf.jvm.bytecode.ArrayStoreInstruction;

import cz.cuni.mff.d3s.stdynpor.Configuration;


public class ArraySchedulerFactory extends DefaultSchedulerFactory 
{
	private static StringSetMatcher neverBreakMethodPatterns;

	private static boolean enabledBreakArrays;

	
	public ArraySchedulerFactory(Config config, VM vm, SystemState ss)
	{
		super(config, vm, ss);
		
		String[] patterns = config.getStringArray("vm.por.array_boundaries.never");
		if (patterns != null) neverBreakMethodPatterns = new StringSetMatcher(patterns);

		enabledBreakArrays = config.getBoolean("cg.threads.break_arrays", false);
	}
	
	public ChoiceGenerator<ThreadInfo> createSharedArrayAccessCG(ElementInfo ei, ThreadInfo ti) 
	{
		// the array object (ei) is shared between multiple threads
		
		if ( ! enabledBreakArrays ) return null;

		if (ss.isAtomic()) return null;
		
		// we need to check just this case here
		if ( ( ! Configuration.usingArrayAccess ) && ( ! Configuration.usingArrayElements ) )
		{
			if (isNeverBreakArrayElementAccess(ti.getPC())) return null;
		}
		
		// the default option is to make a new thread choice
		return getSyncCG(SHARED_ARRAY_ACCESS, ei, ti);		
	}

	/**
	 * Filtering thread choices in library methods (java.*, javax.*).
	 */
	public static boolean isNeverBreakArrayElementAccess(Instruction insn)
	{
		if ( ! (insn instanceof ArrayElementInstruction) ) return false;

		if ( ! enabledBreakArrays ) return true;

		if (neverBreakMethodPatterns != null)
		{
			MethodInfo mth = insn.getMethodInfo();
			if (mth != null)
			{
				String mthName = mth.getFullName();
				
				if (neverBreakMethodPatterns.matchesAny(mthName)) return true;
			}
		}
		
		return false;
	}
}
