/*
 * Copyright (C) 2015, Charles University in Prague.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package cz.cuni.mff.d3s.stdynpor.jpf.bytecode;

import gov.nasa.jpf.vm.ElementInfo;
import gov.nasa.jpf.vm.ThreadInfo;
import gov.nasa.jpf.vm.MethodInfo;

import cz.cuni.mff.d3s.stdynpor.Configuration;
import cz.cuni.mff.d3s.stdynpor.AnalysisResultProcessor;
import cz.cuni.mff.d3s.stdynpor.jpf.JPFUtils;
import cz.cuni.mff.d3s.stdynpor.jpf.ArraySchedulerFactory;


public class BALOAD extends gov.nasa.jpf.jvm.bytecode.BALOAD 
{
	@Override
	protected boolean isNewPorBoundary(ElementInfo arrayObjEI, ThreadInfo curTh)
	{
		if (ArraySchedulerFactory.isNeverBreakArrayElementAccess(this)) return false;
		 
		MethodInfo mi = getMethodInfo();
		String curMethodSig = JPFUtils.getMethodSignature(mi);
			
		String arrayObjClassName = JPFUtils.getArrayClassName(arrayObjEI);
		
		int arrayDimensions = JPFUtils.getArrayDimensionsCount(arrayObjEI);
		
		int elementIndex = curTh.getTopFrame().peek();
		
		// check whether array object is really shared (i.e., reachable from multiple threads)
		if ( ! super.isNewPorBoundary(arrayObjEI, curTh) ) return false;
			
		if ( Configuration.usingArrayAccess )
		{
			// check whether some other thread may access this array object later during program execution (from the current state) in a conflicting way
			
			if ( ! AnalysisResultProcessor.existsConflictingArrayWrite(curTh, curMethodSig, getPosition(), arrayObjClassName, arrayObjEI.getObjectRef(), arrayDimensions, elementIndex) )
			{
				// we do not have to make a thread choice
				return false;
			}
		}
		
		return true;
	}
}
