/*
 * Copyright (C) 2015, Charles University in Prague.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package cz.cuni.mff.d3s.stdynpor.jpf.tools;

import java.util.Set;
import java.util.HashSet;
import java.util.TreeSet;

import gov.nasa.jpf.Config;
import gov.nasa.jpf.ListenerAdapter;
import gov.nasa.jpf.JPF;
import gov.nasa.jpf.search.Search;
import gov.nasa.jpf.vm.VM;
import gov.nasa.jpf.vm.ChoiceGenerator;
import gov.nasa.jpf.vm.ClassInfo;
import gov.nasa.jpf.vm.ElementInfo;
import gov.nasa.jpf.vm.MethodInfo;
import gov.nasa.jpf.vm.ThreadInfo;
import gov.nasa.jpf.vm.Instruction;
import gov.nasa.jpf.vm.choice.ThreadChoiceFromSet;
import gov.nasa.jpf.jvm.bytecode.FieldInstruction;
import gov.nasa.jpf.jvm.bytecode.ArrayElementInstruction;


public class ThreadChoiceGenMonitor extends ListenerAdapter 
{
	private Set<String> fieldThreadChoices = new TreeSet<String>();
	private Set<String> arrayThreadChoices = new TreeSet<String>();
	
	
	public ThreadChoiceGenMonitor(Config config, JPF jpf) 
	{
	}
	
	public void choiceGeneratorRegistered(VM vm, ChoiceGenerator<?> nextCG, ThreadInfo curTh, Instruction insn) 
	{
		if ( ! (nextCG instanceof ThreadChoiceFromSet) ) return;

		if (insn == null) return;
		
		if ( ! ( (insn instanceof FieldInstruction) || (insn instanceof ArrayElementInstruction) ) ) return;
		
		StringBuffer thChoiceInfo = new StringBuffer();

		MethodInfo mi = insn.getMethodInfo();
		if (mi == null) return;
		
		ClassInfo mci = mi.getClassInfo();
		if (mci == null) return;

		thChoiceInfo.append(curTh.getId());
		thChoiceInfo.append(": ");
		thChoiceInfo.append(mci.getName());
		thChoiceInfo.append(".");
		thChoiceInfo.append(mi.getUniqueName());
		thChoiceInfo.append("[");
		thChoiceInfo.append(insn.getPosition());
		thChoiceInfo.append("]");
	
		if (insn instanceof FieldInstruction)
		{
			FieldInstruction fieldInsn = (FieldInstruction) insn;
			
			thChoiceInfo.append(": ");
			thChoiceInfo.append(fieldInsn.getFieldInfo().getFullName());
		}
		
		if (insn instanceof ArrayElementInstruction)
		{
			ArrayElementInstruction arrayInsn = (ArrayElementInstruction) insn;
			
			// array object was probably retrieved by some previous getfield
			
			thChoiceInfo.append(": ");
			thChoiceInfo.append("@");
			thChoiceInfo.append(arrayInsn.getArrayRef(curTh));
			thChoiceInfo.append("[");
			thChoiceInfo.append(arrayInsn.getIndex(curTh));
			thChoiceInfo.append("]");
		}
		
		// program counter of all threads
		for (ThreadInfo th : vm.getThreadList())
		{
			Instruction thpcInsn = th.getPC();
			if (thpcInsn == null) continue;
			
			MethodInfo thpcMI = thpcInsn.getMethodInfo();
			if (thpcMI == null) continue;
			
			ClassInfo thpcMCI = thpcMI.getClassInfo();
			if (thpcMCI == null) continue;
	
			thChoiceInfo.append(" # ");
			thChoiceInfo.append(th.getId());
			thChoiceInfo.append(":");
			thChoiceInfo.append(thpcMCI.getName());
			thChoiceInfo.append(".");
			thChoiceInfo.append(thpcMI.getUniqueName());
			thChoiceInfo.append("[");
			thChoiceInfo.append(thpcInsn.getPosition());
			thChoiceInfo.append("]");
		}

		if (insn instanceof FieldInstruction) fieldThreadChoices.add(thChoiceInfo.toString());
		if (insn instanceof ArrayElementInstruction) arrayThreadChoices.add(thChoiceInfo.toString());
	}
	
	public void searchFinished(Search search) 
	{
		System.out.println("[THREAD CHOICES (FIELD ACCESS)]");
		
		for (String thChoiceStr : fieldThreadChoices)
		{
			System.out.println(thChoiceStr);
		}
		
		System.out.println("[THREAD CHOICES (ARRAY ACCESS)]");
		
		for (String thChoiceStr : arrayThreadChoices)
		{
			System.out.println(thChoiceStr);
		}
	}	

}

